# fmt: off
from loguru import logger

from .browser import get_browser, get_new_page, shutdown_browser
from .data_source import (capture_element, html_to_pic, template_to_html,
                          template_to_pic, text_to_pic)

# fmt:on


async def init(**kwargs):
    """Start Browser

    Returns:
        Browser: Browser
    """
    browser = await get_browser(**kwargs)
    logger.info("Browser Started.")
    return browser


async def shutdown():
    await shutdown_browser()
    logger.info("Browser Stopped.")


browser_init = init

all = [
    "browser_init",
    "text_to_pic",
    "get_new_page",
    "template_to_html",
    "template_to_pic",
    "html_to_pic",
    "capture_element",
]



import html
import re
import traceback
from datetime import datetime

from loguru import logger

from .command_select import select_command
from .data_source import servers
from .model import Hikari_Model
from .moudle.wws_info import get_AccountInfo
from .moudle.wws_recent import get_RecentInfo
from .utils import match_keywords


async def analyze_command(hikari: Hikari_Model) -> Hikari_Model:
    try:
        if hikari.Status == "init":  # 状态为init时才解析
            if not hikari.Input.Command_Text:
                return hikari.error("请发送wws help查看帮助")
            hikari.Input.Command_Text = html.unescape(str(hikari.Input.Command_Text)).strip()
            hikari = await extract_with_special_name(hikari)
            hikari.Function, hikari.Input.Command_List = await select_command(hikari.Input.Command_List)
            hikari = await extract_with_me_or_at(hikari)
            hikari = await extract_with_function(hikari)
        return hikari
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error("解析指令时发生错误，请确认输入参数无误")


async def extract_with_special_name(hikari: Hikari_Model) -> Hikari_Model:
    try:
        match = re.search(r"(\(|（)(.*?)(\)|）)", hikari.Input.Command_Text)  # 是否存在（），存在则需提取出来
        if match:
            hikari.Input.AccountName = match.group(2)
            hikari.Input.Command_List = hikari.Input.Command_Text.replace(match.group(0), "").split()
        else:
            hikari.Input.Command_List = hikari.Input.Command_Text.split()
        return hikari
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error("解析指令时发生错误，请确认输入参数无误")


async def extract_with_me_or_at(hikari: Hikari_Model) -> Hikari_Model:
    try:
        for i in hikari.Input.Command_List:
            if str(i).lower() == "me":
                hikari.Input.Search_Type = 1
                hikari.Input.Platform = hikari.UserInfo.Platform
                hikari.Input.PlatformId = hikari.UserInfo.PlatformId
                hikari.Input.Command_List.remove(i)
                break
            match = re.search(r"CQ:at,qq=(\d+)", i)
            if match:
                hikari.Input.Search_Type = 2
                hikari.Input.Platform = hikari.UserInfo.Platform
                hikari.Input.Platform = str(match.group(1))
                hikari.Input.Command_List.remove(i)
                break
        return hikari
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error("解析指令时发生错误，请确认输入参数无误")


async def extract_with_function(hikari: Hikari_Model) -> Hikari_Model:
    try:
        if hikari.Function in [get_AccountInfo, get_RecentInfo]:
            if datetime.now().hour < 7:
                hikari.Input.Recent_Day = 1
            for i in hikari.Input.Command_List:
                if str(i).isdigit() and len(i) <= 3:
                    hikari.Input.Recent_Day = int(i)
                    hikari.Input.Command_List.remove(i)
            if hikari.Input.Search_Type == 3:
                if len(hikari.Input.Command_List) == 2:
                    hikari.Input.Server, hikari.Input.Command_List = await match_keywords(
                        hikari.Input.Command_List, servers
                    )
                    if hikari.Input.Server:
                        hikari.Input.AccountName = str(hikari.Input.Command_List[0])
                        hikari.Input.Command_List.remove(hikari.Input.Command_List[0])
                    else:
                        return hikari.error("服务器参数输入错误")
                else:
                    return hikari.error("您似乎准备用游戏昵称查询水表，请检查参数中是否包含服务器和游戏昵称，以空格区分")
        return hikari
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error("解析指令时发生错误，请确认输入参数无误")



import traceback
from asyncio.exceptions import TimeoutError

import orjson
from httpx import ConnectTimeout
from loguru import logger

from ..HttpClient_Pool import get_client_yuyuko
from ..model import Hikari_Model
from .publicAPI import check_yuyuko_cache, get_AccountIdByName


async def get_AccountInfo(hikari: Hikari_Model) -> Hikari_Model:
    try:
        if hikari.Status == "init":
            if hikari.Input.Search_Type == 3:
                hikari.Input.AccountId = await get_AccountIdByName(hikari.Input.Server, hikari.Input.AccountName)
                if not isinstance(hikari.Input.AccountId, int):
                    return hikari.error(f"{hikari.Input.AccountId}")
        else:
            return hikari.error("当前请求状态错误")
        is_cache = await check_yuyuko_cache(hikari.Input.Server, hikari.Input.AccountId)
        if is_cache:
            logger.success("上报数据成功")
        else:
            logger.success("跳过上报数据，直接请求")
        url = "https://api.wows.shinoaki.com/public/wows/account/user/info"
        if hikari.Input.Search_Type == 3:
            params = {"server": hikari.Input.Server, "accountId": hikari.Input.AccountId}
        else:
            params = {"server": hikari.Input.Platform, "accountId": hikari.Input.PlatformId}
        client_yuyuko = await get_client_yuyuko()
        resp = await client_yuyuko.get(url, params=params, timeout=None)
        result = orjson.loads(resp.content)
        logger.success(f"本次请求总耗时{resp.elapsed.total_seconds()*1000}，服务器计算耗时:{result['queryTime']}")
        hikari.Output.Yuyuko_Code = result["code"]
        if result["code"] == 200 and result["data"]:
            hikari.Output.Template = "wws-info.html"
            hikari.Output.Width = 920
            hikari.Output.Height = 1000
            return hikari.success(result["data"])
        elif result["code"] == 403:
            return hikari.failed(f"{result['message']}\n请先绑定账号")
        elif result["code"] == 500:
            return hikari.failed(f"{result['message']}\n这是服务器问题，请联系雨季麻麻")
        else:
            return hikari.failed(f"{result['message']}")
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return hikari.erroe("请求超时了，请过会儿再尝试哦~")
    except Exception:
        logger.error(traceback.format_exc())
        return hikari.error("wuwuwu出了点问题，请联系麻麻解决")



from typing import Optional

import httpx
from httpx import AsyncClient, Request, Response
from loguru import logger

from .config import hikari_config


async def before_request(request: Request):
    logger.info(f"{request.method} {request.url}")


async def after_response(response: Response):
    logger.info(f"本次响应的状态码:{response.status_code} {response.http_version} {response.request}")


_client_yuyuko: AsyncClient = None
_client_wg: AsyncClient = None
_client_default: AsyncClient = None


async def create_client_yuyuko() -> AsyncClient:
    global _client_yuyuko
    _client_yuyuko = httpx.AsyncClient(
        headers={
            'Authorization': hikari_config.token,
            "accept": "application/json",
            "Content-Type": "application/json",
        },
        event_hooks={
            "request": [
                before_request,
            ],
            "response": [
                after_response,
            ],
        },
        http2=hikari_config.http2,
    )
    return _client_yuyuko


async def create_client_wg() -> AsyncClient:
    if hikari_config.proxy:
        proxy = {"https://": hikari_config.proxy}
    else:
        proxy = {}
    global client_wg
    _client_wg = httpx.AsyncClient(proxies=proxy)
    return _client_wg


async def create_client_default() -> AsyncClient:
    global _client_default
    _client_default = httpx.AsyncClient()
    return _client_default


async def get_client_yuyuko() -> AsyncClient:
    return _client_yuyuko if _client_yuyuko else await create_client_yuyuko()


async def get_client_wg() -> AsyncClient:
    return _client_wg if _client_wg else await create_client_wg()


async def get_client_default() -> AsyncClient:
    return _client_default if _client_default else await create_client_default()



from dataclasses import dataclass
from typing import List, Protocol, Tuple, runtime_checkable

# from .game.ban_search import get_BanInfo
# from .game.box_check import check_christmas_box
# from .game.ocr import get_Random_Ocr_Pic
# from .game.roll import roll_ship
# from .game.sx import get_sx_info
# from .moudle.publicAPI import get_ship_name
# from .moudle.wws_bind import (change_BindInfo, delete_BindInfo, get_BindInfo,
#                              set_BindInfo, set_special_BindInfo)
# from .moudle.wws_clan import get_ClanInfo
from .moudle.wws_info import get_AccountInfo
from .moudle.wws_recent import get_RecentInfo

# from .moudle.wws_record import get_record
# from .moudle.wws_ship import get_ShipInfo, get_ShipInfoRecent
# from .moudle.wws_shiprank import get_ShipRank


@runtime_checkable
class Func(Protocol):
    async def __call__(self, **kwargs):
        ...


@dataclass
class command:
    keywords: Tuple[str, ...]
    func: Func
    default_func: Func = None


first_command_list = [  # 同指令中越长的匹配词越靠前
    # command(("切换绑定", "更换绑定", "更改绑定"), change_BindInfo),
    # command(("查询绑定", "绑定查询", "绑定列表", "查绑定"), get_BindInfo),
    # command(("删除绑定",), delete_BindInfo),
    # command(("特殊绑定",), set_special_BindInfo),
    # command(("ship.rank", "rank"), get_ShipRank),
    # command(("bind", "绑定", "set"), set_BindInfo),
    command(("recent", "近期"), None, get_RecentInfo),
    # command(("ship","单船",),None,get_ShipInfo,),
    # command(("record", "历史记录"), None, get_record),
    # command(("clan", "军团", "公会", "工会"), None, get_ClanInfo),
    # command(("随机表情包",), get_Random_Ocr_Pic),
    # command(("roll", "随机"), roll_ship),
    # command(("sx", "扫雪"), get_sx_info),
    # command(("ban","封号记录"),get_BanInfo),
    # command(("box", "sd", "圣诞船池"), check_christmas_box),
    # command(("搜船名", "查船名", "船名"), get_ship_name),
]

second_command_list = [
    # command(("recent","近期",),get_ShipInfoRecent,),
    # command(("ship","单船",),get_ShipInfoRecent,),
    # command(("clan", "军团", "公会", "工会"), get_record),
    # command(("record", "历史记录"), get_record),
]


async def findFunction_and_replaceKeywords(match_list, command_List, default_func) -> Tuple[command, List]:
    for com in command_List:
        for kw in com.keywords:
            for i, match_kw in enumerate(match_list):
                if match_kw.find(kw) + 1:
                    match_list[i] = str(match_kw).replace(kw, "")
                    if match_list[i] == "":  # 为空时才删除，防止未加空格没有被split切割
                        match_list.remove("")
                    return com, match_list
    return command(None, default_func, None), match_list


async def select_command(search_list) -> Tuple[Func, List]:
    command, search_list = await findFunction_and_replaceKeywords(search_list, first_command_list, get_AccountInfo)
    if command.func is None:
        command, search_list = await findFunction_and_replaceKeywords(
            search_list, second_command_list, command.default_func
        )
    return command.func, search_list



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hikari_core', 'hikari_core.Html_Render', 'hikari_core.moudle']

package_data = \
{'': ['*'],
 'hikari_core': ['Template/*'],
 'hikari_core.Html_Render': ['templates/*', 'templates/katex/*']}

install_requires = \
['APScheduler>=3.10.1,<4.0.0',
 'aiofiles>=0.8.0',
 'asyncio>=3.4.3,<4.0.0',
 'httpx[http2]>=0.24.0',
 'jinja2>=3.0.0,<4.0.0',
 'orjson>=3.8.11,<4.0.0',
 'playwright>=1.17.2',
 'pydantic>=1.10.7,<2.0.0']

setup_kwargs = {
    'name': 'hikari-core',
    'version': '0.1.5',
    'description': '',
    'long_description': '# Hikari-core\nSDK for yuyuko API\n',
    'author': 'benx1n',
    'author_email': '1119809439@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author         : yanyongyu
@Date           : 2021-03-12 13:42:43
@LastEditors    : yanyongyu
@LastEditTime   : 2021-11-01 14:05:41
@Description    : None
@GitHub         : https://github.com/yanyongyu
"""
__author__ = "yanyongyu"
# fmt: off
from contextlib import asynccontextmanager
from typing import AsyncIterator, Optional

from loguru import logger
from playwright.async_api import (Browser, Error, Page, Playwright,
                                  async_playwright)

# fmt:on
_browser: Optional[Browser] = None
_playwright: Optional[Playwright] = None


async def init(use_browser, **kwargs) -> Browser:
    global _browser
    global _playwright
    _playwright = await async_playwright().start()
    try:
        _browser = await launch_browser(use_browser, **kwargs)
    except Error:
        await install_browser(use_browser)
        _browser = await launch_browser(use_browser, **kwargs)
    return _browser


async def launch_browser(use_browser, **kwargs) -> Browser:
    assert _playwright is not None, "Playwright 没有安装"
    if use_browser == "firefox":
        logger.info("使用 firefox 启动")
        return await _playwright.firefox.launch(**kwargs)

    else:
        # 默认使用 chromium
        logger.info("使用 chromium 启动")
        return await _playwright.chromium.launch(**kwargs)


async def get_browser(use_browser, **kwargs) -> Browser:
    return _browser if _browser and _browser.is_connected() else await init(use_browser, **kwargs)


@asynccontextmanager
async def get_new_page(use_browser, **kwargs) -> AsyncIterator[Page]:
    browser = await get_browser(use_browser)
    page = await browser.new_page(**kwargs)
    try:
        yield page
    finally:
        await page.close()


async def shutdown_browser():
    global _browser
    global _playwright
    if _browser:
        await _browser.close()
        _browser = None
    if _playwright:
        await _playwright.stop()  # type: ignore
        _playwright = None


async def install_browser(use_browser):
    import os
    import sys

    from playwright.__main__ import main

    logger.info("使用镜像源进行下载")
    os.environ["PLAYWRIGHT_DOWNLOAD_HOST"] = "https://npmmirror.com/mirrors/playwright/"
    success = False

    if use_browser == "firefox":
        logger.info("正在安装 firefox")
        sys.argv = ["", "install", "firefox"]
    else:
        # 默认使用 chromium
        logger.info("正在安装 chromium")
        sys.argv = ["", "install", "chromium"]
    try:
        logger.info("正在安装依赖")
        os.system("playwright install-deps")
        main()
    except SystemExit as e:
        if e.code == 0:
            success = True
    if not success:
        logger.error("浏览器更新失败, 请检查网络连通性")



# fmt: off
import os
import time
import traceback

import jinja2
from apscheduler.schedulers.background import BackgroundScheduler
from loguru import logger
from pydantic import ValidationError

from .analyze import analyze_command
from .config import hikari_config, set_hikari_config
from .data_source import set_render_params, template_path
from .Html_Render import html_to_pic
from .model import Hikari_Model, Input_Model, UserInfo_Model
from .utils import startup

# fmt : on
env = jinja2.Environment(loader=jinja2.FileSystemLoader(template_path), enable_async=True)
env.globals.update(
    time=time,
    abs=abs,
    enumerate=enumerate,
)


async def init_hikari(
    platform: str,
    PlatformId: str,
    command_text: str = None,
) -> Hikari_Model:
    """Hikari初始化，如果进行过set_config，则会覆盖这边的对应配置

    Args:
        platform (str): 平台类型
        PlatformId (str): 平台ID
        command_text (str): 传入指令，不带wws

    Returns:
        Hikari_Model: 可通过Hikari.Output.Data内数据判断是否输出
    """
    try:
        userinfo = UserInfo_Model(Platform=platform, PlatformId=PlatformId)
        input = Input_Model(Command_Text=command_text)
        hikari = Hikari_Model(UserInfo=userinfo, Input=input)
        hikari = await analyze_command(hikari)
        if not hikari.Status == "error" and hikari.Function:
            hikari: Hikari_Model = await hikari.Function(hikari)
            if hikari.Output.Data_Type == """<class 'str'>""":
                logger.info(hikari.Output.Data)
                return hikari
            else:
                if hikari_config.auto_rendering:
                    template = env.get_template(hikari.Output.Template)
                    template_data = await set_render_params(hikari.Output.Data)
                    content = await template.render_async(template_data)
                    if hikari_config.auto_image:
                        hikari.success(
                            await html_to_pic(
                                content,
                                wait=0,
                                viewport={"width": hikari.Output.Width, "height": hikari.Output.Height},
                                use_browser=hikari_config.use_broswer,
                            )
                        )
                logger.info(hikari.Output.Data_Type)
        if hikari.Output.Data_Type == """<class 'str'>""":
            logger.info(hikari.Output.Data)
        return hikari
    except ValidationError:
        logger.error(traceback.format_exc())
        return Hikari_Model().error("参数校验错误，请联系开发者确认入参是否符合Model")
    except Exception:
        logger.error(traceback.format_exc())
        return Hikari_Model().error("Hikari-core顶层错误，请检查log")


#startup()
#scheduler = BackgroundScheduler(timezone="Asia/Shanghai")
#scheduler.add_job(startup, 'cron', hour=4)
#scheduler.start()

logger.add(
    "hikari-core-logs/error.log",
    rotation="00:00",
    retention="1 week",
    diagnose=False,
    level="ERROR",
    encoding="utf-8",
)
logger.add(
    "hikari-core-logs/info.log",
    rotation="00:00",
    retention="1 week",
    diagnose=False,
    level="INFO",
    encoding="utf-8",
)
logger.add(
    "hikari-core-logs/warning.log",
    rotation="00:00",
    retention="1 week",
    diagnose=False,
    level="WARNING",
    encoding="utf-8",
)
